@@ -12,6 +12,7 @@ import os
 import sys
 import time
 
+from .library import SYSTEM_GLOBAL_INCLUDES_NAME
 from .model import lint_script
 from .options import fetch_read_write, log_stdout, url_file_relative
 from .parser import parse_expression, parse_script
@@ -30,7 +31,8 @@ def main(argv=None):
     parser.add_argument('-c', '--code', action=_InlineScriptAction, help='execute the BareScript code')
     parser.add_argument('-d', '--debug', action='store_true', help='enable debug mode')
     parser.add_argument('-m', '--markdown-up', action='store_true', help='run with MarkdownUp stubs')
-    parser.add_argument('-s', '--static', action='store_true', help='perform static analysis')
+    parser.add_argument('-s', '--static', dest='static', action='store_const', const='s', help='perform static analysis')
+    parser.add_argument('-x', '--staticx', dest='static', action='store_const', const='x', help='perform static analysis with execution')
     parser.add_argument('-v', '--var', nargs=2, action='append', metavar=('VAR', 'EXPR'), default = [],
                         help='set a global variable to an expression value')
     args = parser.parse_args(args=argv)
@@ -48,11 +50,18 @@ def main(argv=None):
 
         # Get the scripts to run
         scripts = args.scripts
+        ix_user_script = 0
         if args.markdown_up:
             scripts = [('code', 'include <markdownUp.bare>'), *scripts]
+            ix_user_script = 1
+
+            # Add unittest.bare argument globals
+            globals_['vUnittestReport'] = True
+            if args.static:
+                globals_['vUnittestDisabled'] = True
 
         # Parse and execute all source files in order
-        for script_type, script_value in scripts:
+        for ix_script, [script_type, script_value] in enumerate(scripts):
             # Get the script source
             if script_type == 'file':
                 script_name = script_value
@@ -65,49 +74,59 @@ def main(argv=None):
                     raise ValueError(f'Failed to load "{script_value}"')
             else:
                 inline_count += 1
-                script_name = f'<string{inline_count if inline_count > 1 else ""}>'
+                inline_display = inline_count - ix_user_script
+                script_name = f'<string{inline_display if inline_display > 1 else ""}>'
                 script_source = script_value
 
             # Parse the script source
             script = parse_script(script_source, 1, script_name)
 
-            # Run the bare-script linter?
-            if args.static or args.debug:
-                warnings = lint_script(script)
-                if not warnings:
-                    print(f'BareScript: Static analysis "{script_name}" ... OK')
+            # Execute?
+            static_globals = None
+            if args.static != 's':
+                # Set the globals to use for static analysis (below)
+                if not args.static or ix_script < ix_user_script:
+                    static_globals = globals_
                 else:
-                    print(f'BareScript: Static analysis "{script_name}" ... {len(warnings)} warning{"s" if len(warnings) > 1 else ""}:')
+                    # Copy global to keep each script as isolated as possible
+                    static_globals = dict(globals_)
+                    global_includes = static_globals.get(SYSTEM_GLOBAL_INCLUDES_NAME)
+                    if global_includes is not None and isinstance(global_includes, dict):
+                        static_globals[SYSTEM_GLOBAL_INCLUDES_NAME] = dict(global_includes)
+
+                # Execute the script
+                time_begin = time.time()
+                result = execute_script(script, {
+                    'debug': args.debug or False,
+                    'fetchFn': _fetch_include,
+                    'globals': static_globals,
+                    'logFn': log_stdout,
+                    'systemPrefix': _FETCH_INCLUDE_PREFIX,
+                    'urlFn': partial(url_file_relative, script_value) if script_type == 'file' else None
+                })
+                if isinstance(result, (int, float)) and int(result) == result and 0 <= result <= 255:
+                    status_code = int(result) or status_code
+                else:
+                    status_code = (1 if value_boolean(result) else 0) or status_code
+
+                # Log script execution end with timing
+                if args.debug and ix_script >= ix_user_script:
+                    time_end = time.time()
+                    print(f'BareScript executed in {1000 * (time_end - time_begin):.1f} milliseconds')
+
+            # Run the bare-script linter?
+            if args.static and ix_script >= ix_user_script:
+                warnings = lint_script(script, static_globals)
+                if not warnings:
+                    print(f'BareScript static analysis "{script_name}" ... OK')
+                else:
+                    print(f'BareScript static analysis "{script_name}" ... {len(warnings)} warning{"s" if len(warnings) > 1 else ""}:')
                     for warning in warnings:
-                        print(f'BareScript:     {warning}')
-                    if args.static:
-                        status_code = 1
-                        break
-            if args.static:
-                continue
-
-            # Execute the script
-            time_begin = time.time()
-            result = execute_script(script, {
-                'debug': args.debug or False,
-                'fetchFn': _fetch_include,
-                'globals': globals_,
-                'logFn': log_stdout,
-                'systemPrefix': _FETCH_INCLUDE_PREFIX,
-                'urlFn': partial(url_file_relative, script_value) if script_type == 'file' else None
-            })
-            if isinstance(result, (int, float)) and int(result) == result and 0 <= result <= 255:
-                status_code = int(result)
-            else:
-                status_code = 1 if value_boolean(result) else 0
-
-            # Log script execution end with timing
-            if args.debug:
-                time_end = time.time()
-                print(f'BareScript: Script executed in {1000 * (time_end - time_begin):.1f} milliseconds')
+                        print(warning)
+                    status_code = 1
 
             # Stop on error status code
-            if status_code != 0:
+            if status_code != 0 and not args.static:
                 break
 
     except Exception as exc:
