@@ -319,21 +319,24 @@ def validate_expression(expr):
     return validate_type(BARE_SCRIPT_TYPES, 'Expression', expr)
 
 
-def lint_script(script):
+def lint_script(script, globals_=None):
     """
     Lint a BareScript script model
 
     :param script: The `BareScript model <./model/#var.vName='BareScript'>`__
     :type script: dict
+    :param globals_: The script global variables
+    :type globals_: dict or None, optional
     :return: The list of lint warnings
     :rtype: list[str]
     """
 
     warnings = []
+    statements = script['statements']
 
     # Empty script?
     if len(script['statements']) == 0:
-        warnings.append('Empty script')
+        _lint_script_warning(warnings, script, None, 'Empty script')
 
     # Variable used before assignment?
     var_assigns = {}
@@ -341,15 +344,19 @@ def lint_script(script):
     _get_variable_assignments_and_uses(script['statements'], var_assigns, var_uses)
     for var_name in sorted(var_assigns.keys()):
         if var_name in var_uses and var_uses[var_name] <= var_assigns[var_name]:
-            warnings.append(
-                f'Global variable "{var_name}" used (index {var_uses[var_name]}) before assignment (index {var_assigns[var_name]})'
-            )
+            _lint_script_warning(warnings, script, statements[var_uses[var_name]], f'Global variable "{var_name}" used before assignment')
+
+    # Unknown global variable?
+    if globals_ is not None:
+        for var_name in sorted(var_uses.keys()):
+            if var_name not in var_assigns and var_name not in globals_ and var_name not in _BUILTIN_GLOBALS:
+                _lint_script_warning(warnings, script, statements[var_uses[var_name]], f'Unknown global variable "{var_name}"')
 
     # Iterate global statements
     functions_defined = {}
     labels_defined = {}
     labels_used = {}
-    for ix_statement, statement in enumerate(script['statements']):
+    for ix_statement, statement in enumerate(statements):
         statement_key = next(iter(statement.keys()))
 
         # Function definition checks
@@ -358,7 +365,7 @@ def lint_script(script):
 
             # Function redefinition?
             if function_name in functions_defined:
-                warnings.append(f'Redefinition of function "{function_name}" (index {ix_statement})')
+                _lint_script_warning(warnings, script, statement, f'Redefinition of function "{function_name}"')
             else:
                 functions_defined[function_name] = ix_statement
 
@@ -366,57 +373,69 @@ def lint_script(script):
             fn_var_assigns = {}
             fn_var_uses = {}
             args = statement['function'].get('args')
-            _get_variable_assignments_and_uses(statement['function']['statements'], fn_var_assigns, fn_var_uses)
+            fn_statements = statement['function']['statements']
+            _get_variable_assignments_and_uses(fn_statements, fn_var_assigns, fn_var_uses)
             for var_name in sorted(fn_var_assigns.keys()):
                 # Ignore re-assigned function arguments
                 if args is not None and var_name in args:
                     continue
                 if var_name in fn_var_uses and fn_var_uses[var_name] <= fn_var_assigns[var_name]:
-                    warnings.append(
-                        f'Variable "{var_name}" of function "{function_name}" used (index {fn_var_uses[var_name]}) ' +
-                            f'before assignment (index {fn_var_assigns[var_name]})'
+                    _lint_script_warning(
+                        warnings, script, fn_statements[fn_var_uses[var_name]],
+                        f'Variable "{var_name}" of function "{function_name}" used before assignment'
                     )
 
             # Unused variables?
             for var_name in sorted(fn_var_assigns.keys()):
                 if var_name not in fn_var_uses:
-                    warnings.append(
-                        f'Unused variable "{var_name}" defined in function "{function_name}" (index {fn_var_assigns[var_name]})'
+                    _lint_script_warning(
+                        warnings, script, fn_statements[fn_var_assigns[var_name]],
+                        f'Unused variable "{var_name}" defined in function "{function_name}"'
                     )
 
+            # Unknown global variable?
+            if globals_ is not None:
+                for var_name in sorted(fn_var_uses.keys()):
+                    if var_name not in fn_var_assigns and (args is None or var_name not in args) and \
+                       var_name not in globals_ and var_name not in _BUILTIN_GLOBALS:
+                        _lint_script_warning(
+                            warnings, script, fn_statements[fn_var_uses[var_name]], f'Unknown global variable "{var_name}"'
+                        )
+
             # Function argument checks
             if args is not None:
                 args_defined = set()
                 for arg in args:
                     # Duplicate argument?
                     if arg in args_defined:
-                        warnings.append(f'Duplicate argument "{arg}" of function "{function_name}" (index {ix_statement})')
+                        _lint_script_warning(warnings, script, statement, f'Duplicate argument "{arg}" of function "{function_name}"')
                     else:
                         args_defined.add(arg)
 
                         # Unused argument?
                         if arg not in fn_var_uses:
-                            warnings.append(f'Unused argument "{arg}" of function "{function_name}" (index {ix_statement})')
+                            _lint_script_warning(warnings, script, statement, f'Unused argument "{arg}" of function "{function_name}"')
 
             # Iterate function statements
             fn_labels_defined = {}
             fn_labels_used = {}
-            for ix_fn_statement, fn_statement in enumerate(statement['function']['statements']):
+            for ix_fn_statement, fn_statement in enumerate(fn_statements):
                 fn_statement_key = next(iter(fn_statement.keys()))
 
                 # Function expression statement checks
                 if fn_statement_key == 'expr':
                     # Pointless function expression statement?
                     if 'name' not in fn_statement['expr'] and _is_pointless_expression(fn_statement['expr']['expr']):
-                        warnings.append(f'Pointless statement in function "{function_name}" (index {ix_fn_statement})')
+                        _lint_script_warning(warnings, script, statement, f'Pointless statement in function "{function_name}"')
 
                 # Function label statement checks
                 elif fn_statement_key == 'label':
                     # Label redefinition?
                     fn_statement_label = fn_statement['label']['name']
                     if fn_statement_label in fn_labels_defined:
-                        warnings.append(
-                            f'Redefinition of label "{fn_statement_label}" in function "{function_name}" (index {ix_fn_statement})'
+                        _lint_script_warning(
+                            warnings, script, statement,
+                            f'Redefinition of label "{fn_statement_label}" in function "{function_name}"'
                         )
                     else:
                         fn_labels_defined[fn_statement_label] = ix_fn_statement
@@ -428,25 +447,25 @@ def lint_script(script):
             # Unused function labels?
             for label in sorted(fn_labels_defined.keys()):
                 if label not in fn_labels_used:
-                    warnings.append(f'Unused label "{label}" in function "{function_name}" (index {fn_labels_defined[label]})')
+                    _lint_script_warning(warnings, script, statement, f'Unused label "{label}" in function "{function_name}"')
 
             # Unknown function labels?
             for label in sorted(fn_labels_used.keys()):
                 if label not in fn_labels_defined:
-                    warnings.append(f'Unknown label "{label}" in function "{function_name}" (index {fn_labels_used[label]})')
+                    _lint_script_warning(warnings, script, statement, f'Unknown label "{label}" in function "{function_name}"')
 
         # Global expression statement checks
         elif statement_key == 'expr':
             # Pointless global expression statement?
             if 'name' not in statement['expr'] and _is_pointless_expression(statement['expr']['expr']):
-                warnings.append(f'Pointless global statement (index {ix_statement})')
+                _lint_script_warning(warnings, script, statement, 'Pointless global statement')
 
         # Global label statement checks
         elif statement_key == 'label':
             # Label redefinition?
             statement_label = statement['label']['name']
             if statement_label in labels_defined:
-                warnings.append(f'Redefinition of global label "{statement_label}" (index {ix_statement})')
+                _lint_script_warning(warnings, script, statement, f'Redefinition of global label "{statement_label}"')
             else:
                 labels_defined[statement_label] = ix_statement
 
@@ -457,16 +476,27 @@ def lint_script(script):
     # Unused global labels?
     for label in sorted(labels_defined.keys()):
         if label not in labels_used:
-            warnings.append(f'Unused global label "{label}" (index {labels_defined[label]})')
+            _lint_script_warning(warnings, script, statements[labels_defined[label]], f'Unused global label "{label}"')
 
     # Unknown global labels?
     for label in sorted(labels_used.keys()):
         if label not in labels_defined:
-            warnings.append(f'Unknown global label "{label}" (index {labels_used[label]})')
+            _lint_script_warning(warnings, script, statements[labels_used[label]], f'Unknown global label "{label}"')
 
     return warnings
 
 
+# Builtin global variable names
+_BUILTIN_GLOBALS = set(['false', 'if', 'null', 'true'])
+
+
+# Helper to format static analysis warnings
+def _lint_script_warning(warnings, script, statement, message):
+    script_name = script.get('scriptName', '')
+    lineno = statement[next(iter(statement.keys()))].get('lineNumber', 1) if statement is not None else 1
+    warnings.append(f'{script_name}:{lineno}: {message}')
+
+
 # Helper function to determine if an expression statement's expression is pointless
 def _is_pointless_expression(expr):
     expr_key = next(iter(expr.keys()))
