@@ -8,7 +8,7 @@ The BareScript runtime
 import datetime
 import functools
 
-from .library import COVERAGE_GLOBAL_NAME, DEFAULT_MAX_STATEMENTS, EXPRESSION_FUNCTIONS, SCRIPT_FUNCTIONS
+from .library import COVERAGE_GLOBAL_NAME, DEFAULT_MAX_STATEMENTS, EXPRESSION_FUNCTIONS, SCRIPT_FUNCTIONS, SYSTEM_GLOBAL_INCLUDES_NAME
 from .model import lint_script
 from .options import url_file_relative
 from .parser import parse_script
@@ -131,6 +131,15 @@ def _execute_script_helper(script, statements, options, locals_):
                 elif url_fn is not None:
                     include_url = url_fn(include_url)
 
+                # Already included?
+                global_includes = globals_.get(SYSTEM_GLOBAL_INCLUDES_NAME)
+                if global_includes is None or not isinstance(global_includes, dict):
+                    global_includes = {}
+                    globals_[SYSTEM_GLOBAL_INCLUDES_NAME] = global_includes
+                if global_includes.get(include_url):
+                    continue
+                global_includes[include_url] = True
+
                 # Fetch the URL
                 try:
                     include_text = fetch_fn({'url': include_url}) if fetch_fn is not None else None
@@ -144,20 +153,20 @@ def _execute_script_helper(script, statements, options, locals_):
                 if system_include:
                     include_script['system'] = True
 
-                # Run the bare-script linter?
-                if log_fn is not None and options.get('debug'):
-                    warnings = lint_script(include_script)
-                    if warnings:
-                        warning_prefix = f'BareScript: Include "{include_url}" static analysis...'
-                        log_fn(f'{warning_prefix} {len(warnings)} warning{"s" if len(warnings) > 1 else ""}:')
-                        for warning in warnings:
-                            log_fn(f'BareScript:     {warning}')
-
                 # Execute the include script
                 include_options = options.copy()
                 include_options['urlFn'] = functools.partial(url_file_relative, include_url)
                 _execute_script_helper(include_script, include_script['statements'], include_options, None)
 
+                # Run the bare-script linter?
+                if log_fn is not None and options.get('debug'):
+                    warnings = lint_script(include_script, globals_)
+                    if warnings:
+                        warning_prefix = f'BareScript: Include "{include_url}" static analysis...'
+                        log_fn(f'{warning_prefix} {len(warnings)} warning{"s" if len(warnings) > 1 else ""}:')
+                        for warning in warnings:
+                            log_fn(f'BareScript: {warning}')
+
         # Increment the statement counter
         ix_statement += 1
 
